﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using DG.Tweening; 


public class GamePlayManager : MonoBehaviour {

	static public GamePlayManager singleton;

	[HideInInspector]
	public int currentRound=1;
	private int currentPlayerTurn=1;
	private AudioClip player1Record;
	private AudioClip player2Record;

	private LineRenderer lnR;
	private float[] spectrum ;

	private bool gameTimePaused=true;
	private float startRecordTime=0;

	public int player1Hp=100;
	public int player2Hp=100;

	public int recordTime=5;

	public int maxWaveNbr=10;
	private int maxWavePower=100;

	public int DbQuiet=20;
	public int DbMaxToExpload=80;

	private int dbValue=0;

	public List<int> player1WavesList = new List<int> ();
	public List<int> player2WavesList = new List<int> ();

	public List<GameObject> player1WavesListGameObject = new List<GameObject> ();
	public List<GameObject> player2WavesListGameObject = new List<GameObject> ();

	ReferenceManager rfmg;


	int qSamples = 1024;  	// array size
	float refValue = 0.1f; 	// RMS value for 0 dB
	float rmsValue ;  		// sound level - RMS
	float dbValueFloat;   	// sound level - dB
	float volume = 2; 		// set how much the scale will vary


	#region GamePlay Methods
	private void initTimerUi(){
		string timer = "0"+recordTime.ToString()+".00";
		rfmg.timer1Text.text = timer;
		rfmg.timer2Text.text = timer;
	}

	private void ChangeGraphicsToPlayer1(){
		Image recordBtnImage = rfmg.recordBtn.transform.GetComponent<Image>();

		SpriteState spriteState = new SpriteState ();

		recordBtnImage.sprite = rfmg.recordBtn1NotPressed;

		spriteState.highlightedSprite = rfmg.recordBtn1NotPressed;
		spriteState.pressedSprite = rfmg.recordBtn1Pressed;
		spriteState.disabledSprite = rfmg.recordBtnDisabled;

		rfmg.recordBtn.spriteState = spriteState;
	}

	private void ChangeGraphicsToPlayer2(){
		Image recordBtnImage = rfmg.recordBtn.transform.GetComponent<Image>();

		SpriteState spriteState = new SpriteState ();

		recordBtnImage.sprite = rfmg.recordBtn2NotPressed;

		spriteState.highlightedSprite = rfmg.recordBtn2NotPressed;
		spriteState.pressedSprite = rfmg.recordBtn2Pressed;
		spriteState.disabledSprite = rfmg.recordBtnDisabled;

		rfmg.recordBtn.spriteState = spriteState;
	}

	private void EnableRecordButton(bool enable){
		rfmg.recordBtn.interactable = enable;
	}

	private AudioClip RecordVoice(){
		rfmg.audioSource.clip = Microphone.Start ("Built-in Microphone",false,recordTime,44100);
		while(!(Microphone.GetPosition(null)>0)){}
		rfmg.audioSource.Play ();
		return rfmg.audioSource.clip;
	}

	private void PlayRecordedVoice(){
		rfmg.audioSource.Play ();
	}

	public void StartRecord(){
		startRecordTime=Time.time;
		Debug.Log (startRecordTime);
		ClearNullItemInplayer1WavesListGameObject ();
		ClearNullItemInplayer2WavesListGameObject ();

		StopAllCoroutines ();

		if (currentPlayerTurn == 1) {
			gameTimePaused = false;
			EnableRecordButton (false);
			player1Record = RecordVoice ();
			Invoke ("PauseGame",recordTime);
			Invoke ("OnPlayer1RecordOver",recordTime+1);
			StartCoroutine (GetWavesNumberAndPowerOfPlayer(player1WavesList));
		} else if (currentPlayerTurn == 2){
			gameTimePaused = false;
			EnableRecordButton (false);
			player2Record = RecordVoice ();
			Invoke ("PauseGame",recordTime);
			Invoke ("OnPlayer2RecordOver",recordTime+1);
			StartCoroutine (GetWavesNumberAndPowerOfPlayer(player2WavesList));
		}
	}

	private void PauseGame(){
		UpdateTimers ();
		gameTimePaused = true;
	}

	private void OnPlayer1RecordOver(){
		currentPlayerTurn=2;
		EnableRecordButton (true);
		ChangeGraphicsToPlayer2();
		initTimerUi ();
	}

	private void OnPlayer2RecordOver(){
		currentPlayerTurn=1;
		ChangeGraphicsToPlayer1();
		EnableRecordButton (true);

		TurnOffRecordUI ();

		StartBattle ();
	}

	IEnumerator GetWavesNumberAndPowerOfPlayer(List<int> list){
		int index=0;
		float timeCounter=0;
		while (timeCounter < recordTime + 1) {
			if (dbValue > DbQuiet) {
				if (index == list.Count + 1) {
					list.Add (dbValue);
				} else {
					//choose the big one
//					if (dbValue > list [index - 1]) {
//						list [index - 1] = dbValue;
//					}
					//do the sum
					list [index - 1]+=dbValue;
				}
			} else if (index == list.Count) {
				index++;
			}
			timeCounter += Time.deltaTime; 
			yield return new WaitForEndOfFrame ();
		}
	}

	private void UpdateTimers(){
		float currentTime = recordTime - (Time.time - startRecordTime);

		if(currentTime<0){
			currentTime = 0;
		}

		string timerTxt = (currentTime).ToString ("00.00");

		rfmg.timer1Text.text = timerTxt;
		rfmg.timer2Text.text = timerTxt;
	}

	private void StartBattle(){
		player1WavesList.Sort ();
		player2WavesList.Sort ();

		player1WavesList.Reverse ();
		player2WavesList.Reverse ();

		player1WavesList = LimitWaveList (player1WavesList);
		player2WavesList = LimitWaveList (player2WavesList);

		player1WavesList.Reverse ();
		player2WavesList.Reverse ();

		StartBattleUi ();	// take about 4 sec
		Invoke("SpwanWaves",4);
	}

	List<int> LimitWaveList(List<int> list){
		if(list.Count<=maxWaveNbr){
			return list;
		}


		for(int i=maxWaveNbr;i<list.Count;i++){
			list[0] = list[i] + list[0];
			list.RemoveAt (i);
			i--;
		}

		return list;
	}

	void SpwanWaves(){
		GameObject goP1 = new GameObject ();
		goP1.name = "player1 Waves";

		GameObject goP2 = new GameObject ();
		goP2.name = "player2 Waves";

		//P1 loop
		for(int i=0;i<player1WavesList.Count;i++){
			GameObject clone = Instantiate (rfmg.player1wavePrefab);
			clone.transform.position = new Vector2 (clone.transform.position.x,-6);
			clone.name = "Player1 wave "+i;
			clone.transform.parent = goP1.transform;
			WaveController waveController = clone.GetComponent<WaveController>();
			waveController.InitWaveData (player1WavesList[i],1,i);
			player1WavesListGameObject.Add (clone);
		}

		//P2 loop
		for(int i=0;i<player2WavesList.Count;i++){
			GameObject clone = Instantiate (rfmg.player2wavePrefab);
			clone.transform.position = new Vector2 (clone.transform.position.x,6);
			clone.name = "Player2 wave "+i;
			clone.transform.parent = goP2.transform;
			WaveController waveController = clone.GetComponent<WaveController>();
			waveController.InitWaveData (player2WavesList[i],-1,i);
			player2WavesListGameObject.Add (clone);
		}

		UpdateWavesScaleDependsOnIndex ();
	}

	void StartBattleUi(){
		
		Sequence sequence = DOTween.Sequence();
		sequence.PrependInterval (1);
		sequence.Append(rfmg.startFightText.transform.DOScale (Vector3.one, 0.5f)).AppendInterval(1.5f);
		sequence.Append(rfmg.startFightText.transform.DOScale (Vector3.zero, 0.5f));

		sequence.Append(rfmg.player1Text.transform.DOScale (Vector3.one, 0.5f));
		sequence.Append(rfmg.player2Text.transform.DOScale (Vector3.one, 0.5f));
		sequence.Append(rfmg.player1HPText.transform.DOScale (Vector3.one, 0.5f));
		sequence.Append(rfmg.player2HPText.transform.DOScale (Vector3.one, 0.5f));
	}

	void UpdateHpUI(){
		if (player1Hp >= 0) {
			rfmg.player1HPText.text = player1Hp + "HP";
		} else {
			rfmg.player1HPText.text = "0HP";
		}
		if (player2Hp >= 0) {
			rfmg.player2HPText.text = player2Hp + "HP";
		} else {
			rfmg.player2HPText.text = "0HP";
		}
	}

	void SpectrumUpdate(){
		spectrum = new float[1024];
		rfmg.audioSource.GetSpectrumData( spectrum, 0, FFTWindow.BlackmanHarris );

		for(int i=0;i<lnR.numPositions;i++){
			Vector3 previousScale = lnR.GetPosition (i);
			previousScale.y = spectrum [i] * 10 * dbValue;
			lnR.SetPosition (i, previousScale);
		}
	}

	float GetDbValue(){
		float sum = 0;
		for (int i=0; i < qSamples; i++){
			sum += spectrum[i]*spectrum[i]; 				// sum squared samples
		}
		rmsValue = Mathf.Sqrt(sum/qSamples); 				// rms = square root of average
		dbValueFloat = 20*Mathf.Log10(rmsValue/refValue);	// calculate dB
		if (dbValueFloat < -160) dbValueFloat = -160; 		// clamp it to -160dB min
		return dbValueFloat;
	}

	public void ReInitUI(){
		Time.timeScale = 1;
		Sequence sequence = DOTween.Sequence();

		rfmg.MsgText.text = "Wave "+ currentRound;
		sequence.PrependInterval (1);
		sequence.Append(rfmg.MsgText.transform.DOScale (Vector3.one, 0.5f));
		sequence.AppendInterval (1);
		sequence.Append(rfmg.MsgText.transform.DOScale (Vector3.zero, 0.5f)).OnComplete(ActivateSpectrumLine);

		sequence.Append (rfmg.recordBtn.transform.DOScale (Vector3.one, 0.5f));
		sequence.Append (rfmg.timer1Text.transform.DOScale (Vector3.one, 0.5f));
		rfmg.timer1Text.text = "0"+recordTime.ToString()+".00";
	}

	void ActivateSpectrumLine(){
		rfmg.spectrumLine.SetActive (true);
	}

	void TurnOffRecordUI(){
		Sequence sequence = DOTween.Sequence();
		sequence.Append(rfmg.recordBtn.transform.DOScale (Vector3.zero, 0.5f));
		rfmg.spectrumLine.SetActive (false);
		sequence.Append(rfmg.timer1Text.transform.DOScale (Vector3.zero, 0.5f));
	}

	public void CheckNewWave(){
		ClearNullItemInplayer1WavesListGameObject ();
		ClearNullItemInplayer2WavesListGameObject ();
		UpdateHpUI ();
		if ((player1WavesListGameObject.Count <= 1 && player2WavesListGameObject.Count == 0) || (player1WavesListGameObject.Count == 0 && player2WavesListGameObject.Count <= 1)) {
			if (player1Hp > 0 && player2Hp > 0) {	
				currentRound++;
				ReInitUI ();
				player1WavesList.Clear ();
				player2WavesList.Clear ();
			} else {
				if (player1Hp <= 0) {
					rfmg.MsgText.text = "Player 2 Won !!";
				} else if (player2Hp <= 0) {
					rfmg.MsgText.text = "Player 1 Won !!";
				} else if (player2Hp <= 0 && player1Hp <= 0){
					rfmg.MsgText.text = "Draw !!";
				}
				rfmg.MsgText.transform.DOScale (Vector3.one, 0.5f);
				rfmg.replayBtn.transform.DOScale (Vector3.one, 0.5f);
			}
		}
	}

	public void ClearNullItemInplayer1WavesListGameObject(){
		for(int i=0;i<player1WavesListGameObject.Count;i++){
			if(player1WavesListGameObject[i]==null){
				player1WavesListGameObject.RemoveAt (i);
			}
		}
	}

	public void ClearNullItemInplayer2WavesListGameObject(){
		for(int i=0;i<player2WavesListGameObject.Count;i++){
			if(player2WavesListGameObject[i]==null){
				player2WavesListGameObject.RemoveAt (i);
			}
		}
	}

	void UpdateWavesScaleDependsOnPower(){
		maxWavePower = player1WavesList[0]>player2WavesList[0] ? player1WavesList[0] : player2WavesList[0] ;

		for(int i=0;i<player1WavesList.Count;i++){
			float scalePercent = (float)player1WavesList [i] / (float)maxWavePower;
			player1WavesListGameObject [i].transform.localScale = player1WavesListGameObject [i].transform.localScale * scalePercent;
		}

		for(int i=0;i<player2WavesList.Count;i++){
			float scalePercent = (float)player2WavesList [i] / (float)maxWavePower;
			player2WavesListGameObject [i].transform.localScale = player2WavesListGameObject [i].transform.localScale * scalePercent;
		}
	}

	void UpdateWavesScaleDependsOnIndex(){
		
		float player1WavesStartScale = player1WavesListGameObject [0].transform.localScale.x;

		for(int i=0;i<player1WavesList.Count;i++){
			float player1WavesScale = player1WavesStartScale - (0.03f * i);
			player1WavesListGameObject [i].transform.localScale = new Vector3(1-player1WavesScale,1-player1WavesScale,1-player1WavesScale);
		}

		float player2WavesStartScale = player2WavesListGameObject [0].transform.localScale.x;

		for(int i=0;i<player2WavesList.Count;i++){
			float player2WavesScale = player2WavesStartScale - (0.03f * i);
			player2WavesListGameObject [i].transform.localScale = new Vector3(1-player2WavesScale,1-player2WavesScale,1-player2WavesScale);
		}
	}

	public void UpdateWavesScaleDependsOnPower(GameObject wave,float hp,float oldHP){
		//float scalePercent = hp / oldHP;
		float scalePercent = wave.transform.localScale.x - 0.05f;
		wave.transform.localScale = scalePercent*Vector2.one;
	}

	public void ReplayBtn(){
		Application.LoadLevel (Application.loadedLevel);
	}

	#endregion

	#region Unity Methods
	void Awake(){
		singleton = this;
	}

	void Start(){
		Time.timeScale = 1;
		rfmg = ReferenceManager.singleton;
	
		initTimerUi ();
		lnR = rfmg.spectrumLine.GetComponent<LineRenderer> ();

		rfmg.MsgText.text = "Wave "+ currentRound;
		Sequence sequence = DOTween.Sequence();
		sequence.Append(rfmg.MsgText.transform.DOScale (Vector3.one, 0.5f)).AppendInterval(1);
		sequence.Append(rfmg.MsgText.transform.DOScale (Vector3.zero, 0.5f));
	}

	void Update(){
		if (!gameTimePaused) {
			UpdateTimers ();
		}

		SpectrumUpdate ();
		dbValue = Mathf.RoundToInt(GetDbValue()+96);

	}
	#endregion

}
