﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using DG.Tweening; 

public class WaveController : MonoBehaviour {

	public int hp=1;
	private float speed=200;

	private int direction=1;
	private bool canMove=false;

	private TextMesh hpText;

	Rigidbody2D rg;

	/// <summary>
	/// Inits the wave data.
	/// </summary>
	/// <param name="hp">Hp.</param>
	/// <param name="direction">Direction should be 1 or -1</param>
	public void InitWaveData(int hp,int direction,int index){
		this.hp = hp;
		this.direction = direction;
		Invoke ("CanMove",index * 0.5f);
	}

	public void Start(){
		rg = GetComponent<Rigidbody2D>();
		hpText = transform.GetChild (0).GetComponent<TextMesh>();
		UpdateWaveHpUI ();
	}

	public void FixedUpdate(){
		if (canMove) {
			UpdateWaveHpUI ();
			UpdateMovement ();
		}
	}

	void UpdateMovement(){
		rg.velocity = Vector2.up * speed * direction * Time.deltaTime;
	}

	void CanMove(){
		canMove = true;
	}

	void OnTriggerEnter2D(Collider2D obj){
		if(obj.tag == "Player1Wave" && this.gameObject.tag == "Player2Wave"){
			WaveController waveController = obj.gameObject.GetComponent<WaveController>();
			if (waveController.hp > hp) {
				waveController.hp -= hp;
				GamePlayManager.singleton.UpdateWavesScaleDependsOnPower (obj.gameObject,waveController.hp,hp+waveController.hp);
				hp = 0;
				Destroy (this.gameObject);
			} else if (waveController.hp < hp) {
				hp -= waveController.hp;
				GamePlayManager.singleton.UpdateWavesScaleDependsOnPower (this.gameObject,hp,hp+waveController.hp);
				waveController.hp = 0;
				Destroy (obj.gameObject);
			} else if (waveController.hp == hp) {
				Destroy (obj.gameObject);
				Destroy (this.gameObject);
			}
			GamePlayManager.singleton.ClearNullItemInplayer1WavesListGameObject();
			GamePlayManager.singleton.ClearNullItemInplayer2WavesListGameObject();
			Camera.main.DOShakeRotation (0.1f,new Vector3(0,0,5));
			Time.timeScale = 0.5f;
			Invoke ("StopSlowMotion",0.5f);
		}

		if(obj.tag == "Player1" && gameObject.tag=="Player2Wave"){
			if (GamePlayManager.singleton.player1Hp > 0) {
				GamePlayManager.singleton.player1Hp -= hp;
			} else {
				GamePlayManager.singleton.player1Hp = 0;
			}
			GamePlayManager.singleton.CheckNewWave();
			Destroy (this.gameObject);
		}else if(obj.tag == "Player2" && gameObject.tag=="Player1Wave"){
			if (GamePlayManager.singleton.player2Hp > 0) {
				GamePlayManager.singleton.player2Hp -= hp;
			}else {
				GamePlayManager.singleton.player2Hp = 0;
			}
			GamePlayManager.singleton.CheckNewWave();
			Destroy (this.gameObject);
		}
	}

	void StopSlowMotion(){
		Time.timeScale = 1;
	}

	void UpdateWaveHpUI(){
		if (hpText != null) {
			if (hp > 0) {
				hpText.text = hp.ToString ();
			} else {
				hpText.text = "0";
			}
		}
	}
}
